@@ -6,12 +6,14 @@ import warnings
 
 import matplotlib.pyplot as plt
 from matplotlib import cm
+from functools import wraps
 
 
 __all__ = ('Dataset', 'Data',)
 
 def construct_Dataset_from_dataframe(function):
 
+	@wraps(function)
 	def wrapper(*args, **kwargs):
 		dataframe = function(*args, **kwargs)
 		
@@ -50,6 +52,23 @@ def _check_file_exists(path, filename):
 	else:
 		return False
 
+def _remove_nans_from_set(set_to_remove_from):
+	"""Remove multiple nans from a set. Sometimes in Dataset.summary, it returns many nans when it should return only one. Quick fix for that issue"""
+
+	out = set()
+	for item in set_to_remove_from:
+		try:
+			if np.isnan(item) and np.nan not in out:
+				out.update({np.nan})
+				continue
+			if np.isnan(item):
+				continue
+			out.update({item})
+		except TypeError:
+			out.update({item})
+		
+	return out
+
 class Dataset(pd.DataFrame):
 	"""Dataset class for analysis. 
 
@@ -99,23 +118,24 @@ class Dataset(pd.DataFrame):
 		"""Return a brief summary of the data in your Dataset. 
 
 		Returns:
-			summary (Dict): a summary of the Dataset"""
+			(Dict): a summary of the Dataset"""
 		summary = dict()
 		for column in self.columns:
 			if column == self.pointercolumn:
 				continue
-			summary.update({column: set(self[column].values)})
+			summary.update({column: _remove_nans_from_set(set(self[column].values))})
+
 		return summary
 	
 	@construct_Dataset_from_dataframe
 	def query(*args, **kwargs):
-		"""Query the columns of a DataFrame with a boolean expression.
+		"""Query the columns of a Dataset with a boolean expression.
 
 		Parameters:
 			expr (str): The query string to evaluate. You can refer to variables in the environment by prefixing them with an ‘@’ character like @a + b. You can refer to column names that are not valid Python variable names by surrounding them in backticks. Thus, column names containing spaces or punctuations (besides underscores) or starting with digits must be surrounded by backticks. (For example, a column named “Area (cm^2) would be referenced as Area (cm^2)). Column names which are Python keywords (like “list”, “for”, “import”, etc) cannot be used. For example, if one of your columns is called a a and you want to sum it with b, your query should be `a a` + b.
 		
 		Returns:
-			dataset (Dataset): the result of the query
+			(Dataset): the result of the query
 		"""
 		return pd.DataFrame.query(*args, **kwargs)
 
@@ -131,12 +151,13 @@ class Dataset(pd.DataFrame):
 	def select_index(self, index):
 		"""Return dataset with single index specified.
 
-		args:	
-			index (int or index): index to select
+		args:
+			index (int or index): Index to select
 
 		returns:
 			(Dataset): Single row dataset.
 
+
 		"""
 		return pd.DataFrame(self.iloc[index]).T.reset_index(drop = True)
 
@@ -148,7 +169,7 @@ class Dataset(pd.DataFrame):
 			index (index or array-like): index to be removed
 
 		returns:
-			out (Dataset): updated Dataset
+			(Dataset): updated Dataset
 
 		"""
 		index = np.array([index]).flatten()
@@ -159,7 +180,7 @@ class Dataset(pd.DataFrame):
 		return Dataset(path, meta_data, readfileby=self.readfileby)
 	
 	
-	def remove_nonexistant_files_from_metadata(self):
+	def remove_nonexistent_files_from_metadata(self):
 		"""
 		Remove references to files that do not exist in path.
 		"""
@@ -173,13 +194,16 @@ class Dataset(pd.DataFrame):
 		return self.remove_index(remove_index)
 	
 	def _construct_index_to_path(self, path, initializer):
-		"""construct index_to_path from path provided
+		"""Construct index_to_path from path provided
 		
 		args:
 			path (str or Dict): a path to where the real data lives. if dict, form is {path: [indices of initializer for this path]}  
 			initializer (pandas.DataFrame: meta data. one column must contain a pointer (filename) to where each the real data is stored
 
 		"""
+		if len(self) == 0:
+			warnings.showwarning("No meta data.", UserWarning, '', 0)
+			return {}
 		if type(path) != dict:
 			assert type(path) == str, "path must be dict or str"
 			#set all indices to the single path provided
@@ -248,7 +272,49 @@ class Dataset(pd.DataFrame):
 					ignore_index = True
 				)
 
-		return new_df           
+		return new_df   
+
+	def add_calculated_column(self, column_name, how):
+		"""Add a calculated column to the Dataset.
+		
+		args:
+			column_name (str): The new column name
+			how (function): f(self) -> column data. A function which operates on self (pandas.DataFrame) and returns new column data.
+			
+		returns:
+			(Dataset): Updated Dataset.
+			
+		examples:
+			Convert 25um and 10um to measured areas. This will only work if no other diameters are present in the Dataset.
+			```
+			>>>def how(dataframe):
+					nominal_diameter_to_measured_area_dict = {'25um':190, '10um':60}
+					return [nominal_diameter_to_measured_area_dict[x] for x in dataframe['diameter'].values]
+			>>>dset.add_calculated_column('measured_area_um', how = how)
+			```
+		
+		
+		"""
+		
+		if not hasattr(how, '__call__'):
+			raise TypeError('how must be a function that operates on a DataFrame')
+			
+		new_column_data = how(pd.DataFrame(self))
+		self.add_column(column_name, new_column_data)
+		
+		return self
+	
+	def add_column(self, column_name, column_data):
+		"""Add a column to a Dataset.
+		
+		args:
+			column_name (str): The new column name.
+			column_data (array-like): The data for the column
+		
+		"""
+		path_to_index = _convert_ITP_to_path_to_index(self.index_to_path)
+		self[column_name] = column_data
+		return Dataset(path_to_index, self)        
 
 	def get_data(self, groupby=None, labelby=None,):
 		"""
@@ -259,8 +325,10 @@ class Dataset(pd.DataFrame):
 				labelby (str, label, index or array-like of):  what to label the output data by. This will change 'definition' in output Data class
 
 		returns:
-				data (Data): the data
+				(Data): the data
 		"""
+		if len(self) == 0:
+			raise ValueError('No meta data to return data for!!')
 		pointercolumn = self.pointercolumn
 		readfileby = self.readfileby
 
@@ -337,9 +405,9 @@ def _summarize_data(data):
 		defn = data[index]['definition']
 		for key in defn:
 			try:
-				out[key].update(set({value for value in defn[key]}))
+				out[key].update(_remove_nans_from_set(set({value for value in defn[key]})))
 			except KeyError:
-				out.update({key:set({value for value in defn[key]})})
+				out.update({key:_remove_nans_from_set(set({value for value in defn[key]}))})
 	return out
 		
 class iDataIndexer():
@@ -411,7 +479,7 @@ class Data(dict):
 		An indexer as in pandas .iloc Usage is Data.iloc[index]
 
 		returns:
-			indexer (iDataIndexer): indexer for indexing
+			(iDataIndexer): indexer for indexing
 
 		example:
 
@@ -426,7 +494,7 @@ class Data(dict):
 		"""Return a summary of the definitions in data
 		
 		returns:
-			summary (dict): summary of the data included
+			(dict): summary of the data included
 
 		"""
 		return _summarize_data(self.to_dict().copy())
@@ -449,15 +517,16 @@ class Data(dict):
 					indices.append(index)
 		return indices
 
-	def filter(self, data_condition_function_dict, definition_condition_dict=None):
+	def filter(self, data_condition_function_dict, definition_condition_dict=None, additional_data_keys_to_filter=None):
 		"""Filter the data.
 		
 		args:
 			data_condition_function_dict (dict): a dict with one entry. key is data key, value is function which operates on a single value.
-			definition_condition_dict (dict): a dict specifying specific definition conditions, which when satisfied can allow their data to be operated on. If None, all data will be filtered
+			definition_condition_dict (dict): a dict specifying specific definition conditions, which when satisfied can allow their data to be operated on. If None, all data will be filtered.
+			additional_data_keys_to_filter (str or key or array-like): Additional data keys that you wish to filter based on data_condition_function_dict.
 
 		returns:
-			data (Data): filtered Data
+			(Data): filtered Data
 
 		Example:
 
@@ -467,13 +536,23 @@ class Data(dict):
 		```
 		Filter data corresponding to an amplitude of '500ua' such that data key 'R' contains only values >10.
 		```
-		>>> Data.filter({'R': labmda x: x>10}, {'amplitude':'500ua'}) 
+		>>> Data.filter({'R': lambda x: x>10}, {'amplitude':'500ua'}) 
 		```
-		You may use this method to remove outliers.
+		Filter data based on 'saturation' but also filter the 'switching_time' data.
+		```
+		>>> Data.filter({'saturation': lambda x: x > .003}, additional_data_keys_to_filter='switching_time')
+		``` 
+
+		You may use this method to remove outliers, for example.
 
 		"""
 		if definition_condition_dict == None:
 			definition_condition_dict = {}
+
+		if type(additional_data_keys_to_filter) == type(None):
+			additional_data_keys_to_filter = []
+		else:
+			additional_data_keys_to_filter = np.array([additional_data_keys_to_filter]).flatten()
 		
 		assert len(data_condition_function_dict) <= 1, "more than one condition supplied in data_condition_function_dict. only one is supported at a time."
 		data_function_key = list(data_condition_function_dict)[0]
@@ -487,11 +566,22 @@ class Data(dict):
 		not_sat_indices = all_index - set(sat_indices)
 		
 		func = data_condition_function_dict[data_function_key]
+
+		keys_to_update = [key for key in additional_data_keys_to_filter]
+
+		keys_to_update.append(data_function_key)
+		keys_to_update = keys_to_update[::-1]
+
 		
 		for index in sat_indices:
-			old = self[index]['data'][data_function_key]
-			old_shape = old.shape
-			self[index]['data'].update({data_function_key:old[func(old)]})
+			checker = self[index]['data'][data_function_key]
+			for key in keys_to_update:
+				old = self[index]['data'][key]
+				old_shape = old.shape
+				#ensure that all keys in keys_to_update have the same dimensionality
+				if old_shape != checker.shape:
+					raise ValueError('data corresponding to data_key "{}" does not have the same shape as data corresponding to data_function_key "{}"'.format(key, data_function_key))
+				self[index]['data'].update({key:old[func(checker)]})
 			
 		return Data(self.to_dict())
 
@@ -502,7 +592,7 @@ class Data(dict):
 			condition (dict): key is definition key, value is value to find. Multiple values provided will be joined by logical OR. Multiple keys will be joined with logical AND.
 
 		returns:
-			data (Data): data satisfying condition 
+			(Data): data satisfying condition 
 		
 		example:
 
@@ -549,7 +639,7 @@ class Data(dict):
 			inplace (bool): do operation inplace
 
 		returns:
-			data (Data): averaged data across 0th axis.
+			(Data): averaged data across 0th axis.
 		"""
 		
 		if not inplace:
@@ -566,23 +656,17 @@ class Data(dict):
 				tmp_out[key].update({'data':mean_data})
 			return Data(tmp_out)
 
-		else:
-			for key in self:
-				data = self[key]['data']
-				mean_data = {k:np.mean(data[k], axis = 0) for k in data}
-				self[key].update({'data':mean_data})
-			return Data(self)
-
-	def apply(self, data_function, kwargs_for_function=None, inplace = False):
-		"""Apply data_function to the data in each index.
+	def apply(self, function_on_data, pass_defn = False, kwargs_for_function=None,**kwargs):
+		"""Apply data_function to the data in each index. **kwargs will be passed to data_function.
 
 		args:
-			data_function (function or array-like(function, )): f(dict) -> dict. function operates is passed the data_dict (corresponding to self[index]['data']). if array-like, functions will be applied sequentially
-			kwargs_for_function (dict or array-like(dict, )): kwargs for data_function in order to pass additional arguments to the functions being applied. 
-			inplace (bool): operate in place
+			function_on_data (function): f(dict) -> dict. Function is passed the data_dict (corresponding to self[index]['data']).
+			pass_defn (bool): Whether or not to pass the definition to function_on_data. If True, will be passed with other kwargs. 
+			kwargs (**kwargs): Additional arguments for function_on_data. 
+			kwargs_for_function (dict): Carryover from an older version. This serves as a error catch to help users convert older code. This argument is not supported.
 
 		returns:
-				data (Data): the new data after operating on it
+				(Data): the new data after operating on it
 
 		example:
 
@@ -624,32 +708,44 @@ class Data(dict):
 				'data': {'raw_data': array([1, 4, 9], dtype=int64)}}}
 		```
 		"""
-		data_functions = np.array([data_function]).flatten()
-		if type(kwargs_for_function) == type(None):
-			kwargs_for_functions = [{} for ijk in range(len(data_functions))]
-		else:
-			kwargs_for_functions = np.array([kwargs_for_function]).flatten()
+		if type(kwargs_for_function) != type(None):
+			raise ValueError("kwargs_for_function argument is no longer supported as of version 0.0.7. Pass kwargs for the apply function simply as kwargs in .apply()")
 
-		if len(kwargs_for_functions) != len(data_functions):
-			raise ValueError('kwargs_for_function does not match in count to the number of data_functions supplied')
+		#TODO delete kwargs_for_function argument
 
+			
+
+
+		data_function = function_on_data
 		tmp_out = self.to_dict().copy()
 
-		for data_function, kwargs_for in zip(data_functions, kwargs_for_functions):
-			for key in tmp_out.keys():
-				try:
+		for key in tmp_out.keys():
+			try:
+
+				if pass_defn:
+					#ensure no overlap between passed arguments and definition arguments:
+					overlap = set(kwargs.keys()).intersection(set(tmp_out[key]['definition'].keys()))
+					if len(overlap) != 0:
+						raise ValueError('kwargs passed in both definition and as kwargs in .apply(). Overlapping keys are "{}"'.format(overlap))
+
+					to_pass = tmp_out[key]['definition'].copy()
+					for key in kwargs:
+						to_pass.update({key:kwargs})
 					internal_out = {
 						'definition':tmp_out[key]['definition'],
-						'data':data_function(tmp_out[key]['data'].copy(), **kwargs_for)
+						'data':data_function(tmp_out[key]['data'].copy(), defn = tmp_out[key]['definition'], **to_pass)
+					}
+				else:
+					internal_out = {
+						'definition':tmp_out[key]['definition'],
+						'data':data_function(tmp_out[key]['data'].copy(), **kwargs)
 					}
-					tmp_out.update({key:internal_out})
-				except Exception as e:
-					print('Error in data_function: {} \n{}'.format(data_function.__name__, e))
-					print('Skipping data key: {} with defintion: \n{}'.format(key, tmp_out[key]['definition']))
 
-		if inplace:
-			self.__init__(tmp_out)
-			return
+				tmp_out.update({key:internal_out})
+			except Exception as e:
+				print('Error in data_function: {} \n{}'.format(data_function.__name__, e))
+				print('Skipping data key: {} with defintion: \n{}'.format(key, tmp_out[key]['definition']))
+
 		return Data(tmp_out)
 
 	def to_dict(self):
@@ -658,10 +754,10 @@ class Data(dict):
 
 
 		returns:
-			out (dict): a dict class with identical structure"""
+			(dict): a dict class with identical structure"""
 		return {key: self[key] for key in self.keys()}
 
-	def plot(self, x=None, y=None, ax=None):
+	def plot(self, x=None, y=None, ax=None, color = None, cmap = 'viridis', labelby = None, **kwargs):
 		"""
 		Plot the data. If ax is provided returns ax, otherwise returns fig, ax.
 
@@ -669,6 +765,9 @@ class Data(dict):
 			x (key): data dict key for x axis.
 			y (key or array-like): data dict key for y axis
 			ax (matplotlib.axis): axis to plot on 
+			color (str): Color of plot. (Override colormap)
+			cmap (str): Color map. See matplotlib.cm.cmaps_listed for allowed colormaps.
+			labelby (str): Definition key to use for plot legend.
 
 		returns:
 			fig (matplotlib.figure): figure of plot
@@ -680,8 +779,13 @@ class Data(dict):
 		else:
 			return_fig = False
 
-		
-		colors = [cm.viridis(x) for x in np.linspace(0, 1, len(self.keys()))]
+		if cmap not in set(cm.cmaps_listed.keys()):
+			raise KeyError('cmap "{}" not supported. Supported colormaps are {}'.format(cmap, cm.cmaps_listed.keys()))	
+
+		if color == None:
+			colors = [cm.cmaps_listed[cmap](x) for x in np.linspace(0, 1, len(self.keys()))]
+		else: 
+			colors = [color for i in range(len(self.keys()))]
 
 		for color, index in zip(colors, self.keys()):
 			if x == None:
@@ -690,6 +794,11 @@ class Data(dict):
 				xs = self[index]['data'][x]
 				data_keys_to_plot = set(self[index]['data'].keys()) - set({x})
 
+			if labelby == None:
+				label = None
+			else:
+				label = self[index]['definition'][labelby]
+
 			if type(y) == type(None):
 				pass
 			else:
@@ -701,85 +810,27 @@ class Data(dict):
 				
 				if len(to_plot.shape) == 1: #1d data
 					if x == None:
-						ax.plot(to_plot, color = color)
+						ax.plot(to_plot, color = color, label = label, **kwargs)
 					else:
-						ax.plot(xs, to_plot, color = color)
+						ax.plot(xs, to_plot, color = color, label = label, **kwargs)
 					continue
 					
 				for i in range(to_plot.shape[0]):
-					if x == None:
-						ax.plot(to_plot[i,:], color = color)
+					if i == 0: #label only the first becuase we don't want a big legend with many trials
+						if x == None:
+							ax.plot(to_plot[i,:], color = color, label = label, **kwargs)
+						else:
+							ax.plot(xs[i,:], to_plot[i,:], color = color, label = label, **kwargs)
 					else:
-						ax.plot(xs[i,:], to_plot[i,:], color = color)
+						if x == None:
+							ax.plot(to_plot[i,:], color = color, **kwargs)
+						else:
+							ax.plot(xs[i,:], to_plot[i,:], color = color, **kwargs)
+
+		if labelby != None:
+			ax.legend()
 
 		if return_fig:
 			return fig, ax
 		else:
 			return ax
-
-class dataset(Dataset):
-	"""
-	subclass of Dataset - used to initialize/read from a specific folder. dataset, unlike Dataset will search a folder for meta_data and help create it if it does not exist
-	----
-	path: (str or dict) a path to where the real data lives. if dict, form is {path: [indices of initializer for this path]}  
-	initializer: (pandas.DataFrame) meta_data. one column must contain a pointer (filename) to where each the real data is stored
-
-	"""
-
-	def __init__(self, path, meta_data = None):
-		"""
-		subclass of Dataset - used to initialize/read from a specific folder
-		"""
-		warnings.showwarning('dataset class is deprecated. use load_Dataset() instead', DeprecationWarning, '', 0,)
-		tmp_df = self._build_df(path, meta_data)
-		super().__init__(path, tmp_df)
-
-	def _build_df(self, path, meta_data):
-		if type(meta_data) == type(None):
-			try:
-				return pd.read_pickle(path + 'meta_data')
-			except FileNotFoundError:
-				print('meta_data does not exist in path {} you may want to create it with .generate_meta_data()'.format(path))
-				return pd.DataFrame()
-		else:
-			return meta_data
-
-	def generate_meta_data(self, mapper):
-		"""
-		generate meta_data from a Dataset
-		----
-
-		mapper: (function: filename (str) -> dict) operates on a single file name in order to get the columns (dict key) and values (dict value) for meta_data of that file 
-		"""
-		if not self._is_empty:
-			yn = input('this Dataset already has meta_data, do you wish to recreate it? (y/n)')
-			if yn.lower() != 'y':
-				return
-
-		for file in os.listdir(self.path):
-			try:
-				meta_data = pd.DataFrame(mapper(file), index = [0])
-			except Exception as e:
-				print('unable to process file: {} \nError: {}'.format(file, e))
-				continue
-			try:
-				existing_meta_data = pd.concat([existing_meta_data, meta_data], ignore_index = True)
-			except NameError:
-				existing_meta_data = meta_data.copy()
-
-		if self.pointercolumn not in set(existing_meta_data.columns): 
-			warnings.showwarning('there is no map to key "{}" in mapping function "{}" provided\nEnsure self.pointercolumn property has been set appropriately or else you will be unable to retrieve data'.format(self.pointercolumn, mapper.__name__), SyntaxWarning, '', 0,)
-
-		existing_meta_data.to_pickle(self.path+'meta_data')
-		self.__init__(path = self.path, meta_data = existing_meta_data)
-		return
-
-
-	def print_file_name(self):
-		"""use this function if you wish to print an example file for help building a mapping function for generate_met_data()"""
-		for fname in os.listdir(self.path):
-			if fname == 'meta_data' or fname == '.ipynb_checkpoints':
-				continue
-			else:
-				print(fname)
-				break
\ No newline at end of file
